float4x4 World;
float4x4 View;
float4x4 Projection;

float3 AmbientLightColor;
float3 LightColor[3];
float3 LightDirection[3];
float3 CameraPosition;
float3 CameraDirection;
float Specularity;

texture Texture;
sampler TextureSampler = sampler_state 
{ 
	Texture = (Texture); 
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

struct VertexShaderInput
{
    float4 Position : POSITION0;
    float2 UV : TEXCOORD0;
    float3 Normal : NORMAL0;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float2 UV : TEXCOORD0;
    float3 Normal : TEXCOORD1;
    float3 ViewDirection : TEXCOORD2;
    float3 WorldPosition : TEXCOORD3;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 position = mul(input.Position, World);
    float3 normal = normalize(mul(input.Normal, World));
    float3 viewDirection = CameraDirection - position;
    float3 worldPosition = position / position.w;
    
	position = mul(mul(position, View), Projection);
    
    output.Position = position;
    output.Normal = normal;
    output.ViewDirection = viewDirection;
    output.WorldPosition = worldPosition;
    output.UV = input.UV;

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
    float3 color = AmbientLightColor;
    float4 textureDiffuse = tex2D(TextureSampler, input.UV);
    
    for (int i = 0; i < 3; i++)
    {
		float3 normal = normalize(input.Normal);
		float3 lightDirection = normalize(LightDirection[i]);
		float3 viewDirection = normalize(input.ViewDirection);
		
		float4 difference = saturate(dot(normal, lightDirection));
		
		float3 reflect = normalize(2 * difference * normal - lightDirection);
		float4 specular = pow(saturate(dot(reflect, viewDirection)), Specularity);
		
		color += LightColor[i] * (difference + specular);
    }
    
    color *= textureDiffuse;
    
    return float4(color, 1);
}

technique Technique1
{
    pass Pass1
    {
        // TODO: set renderstates here.

        VertexShader = compile vs_1_1 VertexShaderFunction();
        PixelShader = compile ps_2_0 PixelShaderFunction();
    }
}
